import random
from transformers import pipeline, BertTokenizerFast, BertForTokenClassification


class EntityAnonymizer:
    """
    A class to anonymize named entities in text using a pre-trained NER model.
        
        Initialize the anonymizer with a chosen entity context and language setting.

        Args:
            context (dict): A dictionary containing mappings of sensitive data to their anonymized forms.
                
                Required keys in the dictionary:
                    - 'LABEL_1': list of person names.
                    - 'LABEL_3': list of organization names.
                    - 'LABEL_5': list of location names.
            multilingua (bool): Use multilingual model if True, else monolingual.

        Example:
        >>> anonymizer = EntityAnonymizer(context=MARVEL_ENTITIES)
        >>> anonymizer.anonymize("Angela Merkel visited Microsoft headquarters.")
        """
    def __init__(self, context: dict, multilingual: bool = False):

        self.context = context

        model_name = (
            'aimarbp02/ner-bert-base-multilingual-cased' if multilingua
            else 'aimarbp02/ner-bert-base-cased'
        )
        model = BertForTokenClassification.from_pretrained(model_name)
        tokenizer = BertTokenizerFast.from_pretrained(model_name)

        self.ner_pipeline = pipeline(
            "token-classification",
            model=model,
            tokenizer=tokenizer,
            aggregation_strategy="simple"
        )

    def anonymize(self, text: str) -> str:
        """
        Anonymize named entities in the text based on the loaded context.

        Args:
            text (str): Text to anonymize.

        Returns:
            str: Anonymized text.
        """
        predictions = self.ner_pipeline(text)
        new_text = ""

        for ent in predictions:
            if ent["entity_group"] in self.context:
                new_text += random.choice(self.context[ent["entity_group"]]) + " "
            elif ent["entity_group"] in ['LABEL_0', 'LABEL_7', 'LABEL_8']:
                new_text += ent["word"] + " "

        return new_text.strip()


MARVEL_ENTITIES = {
    "LABEL_1": [
        "Iron Man", "Thor Odinson", "Black Widow", "Spider-Man",
        "Loki Laufeyson", "Wolverine", "Captain America", "Doctor Strange",
        "Scarlet Witch", "Hawkeye", "Star-Lord", "Gamora", "Ant-Man",
        "Deadpool", "Nick Fury", "Vision", "Black Panther", "Captain Marvel"
    ],
    "LABEL_5": [
        "Wakanda", "Asgard", "Knowhere", "New York City", "Sokovia",
        "Kamar-Taj", "Xandar", "The Quantum Realm", "Hell's Kitchen",
        "Latveria", "Savage Land", "Genosha", "Mount Wundagore"
    ],
    "LABEL_3": [
        "S.H.I.E.L.D.", "Hydra", "The Avengers", "X-Men",
        "Stark Industries", "The Eternals", "Fantastic Four",
        "The Illuminati", "The Sinister Six", "A.I.M.",
        "The Ravagers", "Weapon X", "Oscorp"
    ]
}

STARWARS_ENTITIES = {
    "LABEL_1": [
        "Luke Skywalker", "Darth Vader", "Obi-Wan Kenobi",
        "Yoda", "Han Solo", "Leia Organa", "Kylo Ren",
        "Rey Skywalker", "Padmé Amidala", "Anakin Skywalker",
        "Mace Windu", "Count Dooku", "Qui-Gon Jinn"
    ],
    "LABEL_5": [
        "Tatooine", "Hoth", "Dagobah", "Endor", "Naboo",
        "Coruscant", "Mustafar", "Kashyyyk", "Geonosis",
        "Kamino", "Jakku", "Scarif", "Ahch-To"
    ],
    "LABEL_3": [
        "The Jedi Order", "The Sith", "The Galactic Empire",
        "The Rebel Alliance", "The First Order", "The Resistance",
        "The Galactic Republic", "The Trade Federation",
        "Clone Army", "Separatists", "Mandalorians", "New Republic"
    ]
}

HAPPRYPOTTER_ENTITIES = {
    "LABEL_1": [
        "Harry Potter", "Hermione Granger", "Ron Weasley", "Draco Malfoy",
        "Ginny Weasley", "Albus Dumbledore", "Severus Snape", "Sirius Black",
        "Lord Voldemort", "Rubeus Hagrid", "Luna Lovegood", "Neville Longbottom"
    ],
    "LABEL_5": [
        "Hogwarts", "Hogsmeade", "Diagon Alley", "The Forbidden Forest",
        "Godric's Hollow", "The Burrow", "Little Whinging", "Azkaban",
        "Knockturn Alley", "Ministry of Magic", "Shell Cottage"
    ],
    "LABEL_3": [
        "Gryffindor", "Slytherin", "Ravenclaw", "Hufflepuff",
        "Dumbledore's Army", "The Order of the Phoenix",
        "Death Eaters", "Ministry of Magic", "Hogwarts Express",
        "Quidditch Teams", "Magical Congress", "The Daily Prophet"
    ]
}


from setuptools import setup, find_packages

setup(
    name='ner_anon',
    version='0.1.1',
    packages=find_packages(),
    install_requires=[
        'transformers>=4.0.0',
        'torch',  # You might want to pin a version
    ],
    author='Aimar_bp',
    description='A NER-based anonymization tool that replaces named entities in text with random entities from a given context.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.7',
)


from .data import MARVEL_ENTITIES, STARWARS_ENTITIES, HAPPRYPOTTER_ENTITIES
from .anonymizer import EntityAnonymizer

__all__ = [
    "MARVEL_ENTITIES", "STARWARS_ENTITIES", "HAPPRYPOTTER_ENTITIES",
    "EntityAnonymizer"
]